#include "pool.h"
#include <string.h>
#include "debug.h"
 
void pool_init(struct pool *pool){
  unsigned int i;
  
  memset(pool,0,sizeof(pool));

  NO_OCCUPES(pool) = 0;
  NO_LIBRES(pool) = MAX_ENTRIES;
  PREMIER_LIBRE(pool) = 0;
  PREMIER_OCCUPE(pool) = -1;
  
  for(i=0;i< MAX_ENTRIES;i++)
    {
      EST_LIBRE(pool,i) = 1;
      PROC(pool,i) = (i + 1)%MAX_ENTRIES;
      PREC(pool,i) = (i - 1)%MAX_ENTRIES;
    }   
}

void flip_state(struct pool *pool,int i)
{
  EST_LIBRE(pool,i) = ! EST_LIBRE(pool,i);
}

void enlever(struct pool *pool,int tmp)
{
  int prec = PREC(pool,tmp);
  int proc = PROC(pool,tmp);

  if (proc == tmp)
    proc = -1;
  else {
    /* VOIR(NO_OCCUPES(pool)); */
/*     VOIR(proc); */
/*     VOIR(prec); */

    PROC(pool,prec) = proc;
    /* !!!!! */
    /* VOIR(PROC(pool,prec)); */
    /*     VOIR(NO_OCCUPES(pool)); */
   /*  ERREXIT("Le probleme est bien ici"); */
    PREC(pool,proc) = prec;
  }

 
  if(EST_LIBRE(pool,tmp))
    PREMIER_LIBRE(pool)=proc;
  else
    PREMIER_OCCUPE(pool)=proc;
 }

void ajouter(struct pool *pool,int tmp,int i)
{
  int prec = pool->maillot[i].prec;
  int *premier;

  if(i == -1)
    {
      pool->maillot[tmp].proc = 
      pool->maillot[tmp].prec = tmp;
    }
  else
    {
      pool->maillot[tmp].proc = i;
      pool->maillot[tmp].prec = prec;
      pool->maillot[i].prec = tmp;
      pool->maillot[prec].proc = tmp;
    }

  if(EST_LIBRE(pool,tmp))
    premier = &PREMIER_LIBRE(pool);
  else
    premier = &PREMIER_OCCUPE(pool);

  if(*premier == i)
    *premier = tmp;
  
}


POOL_CONTENT_T *
allouer(struct pool *pool){
  
  int tmp;

  
  /* S'approprier du premier libre */
  tmp = PREMIER_LIBRE(pool);
  /* Si il n'y a pas d'espace echec */
  if(tmp == -1)
    return NULL;

  /* Sinon mettre  jour la pool des maillot libres */
    enlever(pool,tmp);
  
  
  NO_LIBRES(pool)--;

  
  /* Changer d'etat  tmp */
  flip_state(pool,tmp);

  
  /* Mettre  jour la pool des occups */
  ajouter(pool,tmp,PREMIER_OCCUPE(pool));
  NO_OCCUPES(pool)++;
  
  


  return &(CONTENU(pool,tmp));
}


int
chercher(struct pool * pool, POOL_CONTENT_T * address)
{
  int i;

  if( (i = PREMIER_OCCUPE(pool)) == -1)
    return -1;

  do{
    if(&CONTENU(pool,i) ==  address)
      return i;
    i = pool->maillot[i].proc;
  }while(i!=PREMIER_OCCUPE(pool));

  return -1;
}


int liberer(struct pool *pool,POOL_CONTENT_T * address)
{
  int i = chercher(pool,address);
  
  if(i == -1)
    return -1;
    
  /* Mettre  jour la pool des maillot occups */
  enlever(pool,i);
  NO_OCCUPES(pool)--;
  
  /* Changer d'etat  i */
  flip_state(pool,i);
  
  /* Mettre  jour la liste des maillot libres */
  ajouter(pool,i,pool->premier_libre);
  NO_LIBRES(pool)++;

  return 0;
}

